import django_tables2 as tables
from netbox.tables import NetBoxTable

from cesnet_service_path_plugin.models import SegmentCircuitMapping


class SegmentCircuitMappingTable(NetBoxTable):
    segment = tables.Column(linkify=True, verbose_name="Segment", orderable=True, order_by=("segment__name",))
    segment_network_label = tables.Column(
        linkify=False,
        verbose_name="Network Label",
        orderable=True,
        order_by=("segment__network_label",),
        accessor="segment.network_label",
    )
    segment_site_a = tables.Column(
        linkify=True,
        verbose_name="Site A",
        orderable=True,
        order_by=("segment__site_a__name",),
        accessor="segment.site_a",
    )
    segment_location_a = tables.Column(
        linkify=True,
        verbose_name="Location A",
        orderable=True,
        order_by=("segment__location_a__name",),
        accessor="segment.location_a",
    )
    segment_site_b = tables.Column(
        linkify=True,
        verbose_name="Site B",
        orderable=True,
        order_by=("segment__site_b__name",),
        accessor="segment.site_b",
    )
    segment_location_b = tables.Column(
        linkify=True,
        verbose_name="Location B",
        orderable=True,
        order_by=("segment__location_b__name",),
        accessor="segment.location_b",
    )
    circuit = tables.Column(linkify=True, verbose_name="Circuit", orderable=True, order_by=("circuit__name",))

    class Meta(NetBoxTable.Meta):
        model = SegmentCircuitMapping
        fields = (
            "id",
            "circuit",
            "segment",
            "segment_network_label",
            "segment_site_a",
            "segment_location_a",
            "segment_site_b",
            "segment_location_b",
            "actions",
        )
        default_columns = (
            "circuit",
            "segment",
            "segment_network_label",
            "segment_site_a",
            "segment_site_b",
        )



from netbox.api.metadata import ContentTypeMetadata
from netbox.api.viewsets import NetBoxModelViewSet

from cesnet_service_path_plugin.models import Segment
from cesnet_service_path_plugin.filtersets import SegmentFilterSet
from cesnet_service_path_plugin.api.serializers import SegmentSerializer, SegmentDetailSerializer


class SegmentViewSet(NetBoxModelViewSet):
    metadata_class = ContentTypeMetadata
    queryset = Segment.objects.all()
    filterset_class = SegmentFilterSet

    def get_serializer_class(self):
        """
        Return appropriate serializer based on action
        """
        if self.action == "retrieve":
            pathdata = self.request.query_params.get("pathdata", "false").lower() == "true"
            if pathdata:
                return SegmentDetailSerializer

        # Use the updated SegmentSerializer for all other actions (including create/update)
        return SegmentSerializer



import django_tables2
from circuits.models import Provider
from circuits.tables import CircuitTable
from django.conf import settings
from django.utils.translation import gettext_lazy as _
from netbox.plugins import PluginTemplateExtension
from utilities.tables import register_table_column

from cesnet_service_path_plugin.models import Segment

plugin_settings = settings.PLUGINS_CONFIG.get("cesnet_service_path_plugin", {})

# Extra Views


class CircuitKomoraSegmentExtension(PluginTemplateExtension):
    models = ["circuits.circuit"]

    def full_width_page(self):
        return self.render(
            "cesnet_service_path_plugin/circuit_segments_extension.html",
        )


class ProviderSegmentExtension(PluginTemplateExtension):
    models = ["circuits.provider"]

    def full_width_page(self):
        return self.render(
            "cesnet_service_path_plugin/provider_segments_extension.html",
        )


class SiteSegmentExtension(PluginTemplateExtension):
    models = ["dcim.site"]

    def full_width_page(self):
        return self.render(
            "cesnet_service_path_plugin/site_segments_extension.html",
        )


class LocationSegmentExtension(PluginTemplateExtension):
    models = ["dcim.location"]

    def full_width_page(self):
        return self.render(
            "cesnet_service_path_plugin/location_segments_extension.html",
        )


class TenantProviderExtension(PluginTemplateExtension):
    models = ["tenancy.tenant"]

    def left_page(self):
        provider = Provider.objects.filter(
            custom_field_data__tenant=self.context["object"].pk
        ).first()

        provider_circuits_count = provider.circuits.count() if provider else None
        provider_segments_count = (
            Segment.objects.filter(provider_id=provider.id).count()
            if provider
            else None
        )

        return self.render(
            "cesnet_service_path_plugin/tenant_provider_extension.html",
            extra_context={
                "provider": provider,
                "provider_circuits_count": provider_circuits_count,
                "provider_segments_count": provider_segments_count,
            },
        )


template_extensions = [
    CircuitKomoraSegmentExtension,
    TenantProviderExtension,
    ProviderSegmentExtension,
    SiteSegmentExtension,
    LocationSegmentExtension,
]

# Extra Columns

circuit_segments = django_tables2.TemplateColumn(
    verbose_name=_("Segments"),
    template_name="cesnet_service_path_plugin/circuit_segments_template_column.html",
    orderable=False,
    linkify=False,
)

register_table_column(circuit_segments, "circuit_segments", CircuitTable)



from circuits.api.serializers import CircuitSerializer
from netbox.api.serializers import NetBoxModelSerializer
from rest_framework import serializers

from cesnet_service_path_plugin.api.serializers.segment import SegmentSerializer
from cesnet_service_path_plugin.models import ServicePath


class ServicePathSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name="plugins-api:cesnet_service_path_plugin-api:servicepath-detail"
    )
    segments = SegmentSerializer(many=True, read_only=True, nested=True)
    circuits = CircuitSerializer(required=False, many=True, nested=True)

    class Meta:
        model = ServicePath
        fields = [
            "id",
            "url",
            "display",
            "name",
            "status",
            "kind",
            "segments",
            "circuits",
            "tags",
        ]
        brief_fields = [
            "id",
            "url",
            "display",
            "name",
            "tags",
        ]

    def validate(self, data):
        super().validate(data)
        return data



# Generated by Django 5.0.6 on 2024-07-11 12:05

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        (
            "cesnet_service_path_plugin",
            "0005_segmentcircuitmapping_segment_circuits_and_more",
        ),
    ]

    operations = [
        migrations.RemoveField(
            model_name="servicepath",
            name="circuits",
        ),
        migrations.DeleteModel(
            name="ServicePathCircuitMapping",
        ),
    ]



from django.db import migrations


def copy_state_to_status(apps, schema_editor):
    ServicePath = apps.get_model("cesnet_service_path_plugin", "ServicePath")

    # Define mapping from state to status
    state_to_status = {
        "active": "active",
        "planned": "planned",
        "decommissioned": "decommissioned",
    }

    for service_path in ServicePath.objects.all():
        # Get the corresponding status value or default to 'active' if not found
        new_status = state_to_status.get(service_path.state.lower(), "active")
        service_path.status = new_status
        service_path.save()


def reverse_copy_state_to_status(apps, schema_editor):
    ServicePath = apps.get_model("cesnet_service_path_plugin", "ServicePath")

    # Define reverse mapping from status to state
    status_to_state = {
        "active": "active",
        "planned": "planned",
        "decommissioned": "decommissioned",
    }

    for service_path in ServicePath.objects.all():
        # Get the corresponding state value or default to 'active' if not found
        old_state = status_to_state.get(service_path.status.lower(), "active")
        service_path.state = old_state
        service_path.save()


class Migration(migrations.Migration):
    dependencies = [
        (
            "cesnet_service_path_plugin",
            "0020_add_field_status_to_segment_and_servicepath",
        ),
    ]

    operations = [
        migrations.RunPython(copy_state_to_status, reverse_copy_state_to_status)
    ]



# Generated by Django 5.0.6 on 2024-07-25 11:03

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("circuits", "0043_circuittype_color"),
        ("cesnet_service_path_plugin", "0008_servicepath_comments"),
        ("tenancy", "0015_contactassignment_rename_content_type"),
    ]

    operations = [
        migrations.RenameField(
            model_name="segment",
            old_name="supplier_segment_contract",
            new_name="provider_segment_contract",
        ),
        migrations.RenameField(
            model_name="segment",
            old_name="supplier_segment_id",
            new_name="provider_segment_id",
        ),
        migrations.RenameField(
            model_name="segment",
            old_name="supplier_segment_name",
            new_name="provider_segment_name",
        ),
        migrations.AddField(
            model_name="segment",
            name="provider",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="circuits.provider",
            ),
        ),
        migrations.AlterField(
            model_name="segment",
            name="supplier",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="tenancy.tenant",
            ),
        ),
    ]



from rest_framework import serializers
from netbox.api.serializers import NetBoxModelSerializer
from cesnet_service_path_plugin.api.serializers.segment import SegmentSerializer
from cesnet_service_path_plugin.api.serializers.service_path import (
    ServicePathSerializer,
)
from cesnet_service_path_plugin.models import ServicePathSegmentMapping


class ServicePathSegmentMappingSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name="plugins-api:cesnet_service_path_plugin-api:servicepathsegmentmapping-detail"
    )
    # service_path = serializers.PrimaryKeyRelatedField(
    #    queryset=ServicePath.objects.all(),
    #    required=True
    # )
    service_path = ServicePathSerializer(nested=True)
    segment = SegmentSerializer(nested=True)

    class Meta:
        model = ServicePathSegmentMapping
        fields = [
            "id",
            "url",
            "display",
            "service_path",
            "segment",
        ]
        brief_fields = [
            "id",
            "url",
            "display",
            "service_path",
            "segment",
        ]

    def validate(self, data):
        super().validate(data)
        return data



from django.urls import include, path
from utilities.urls import get_model_urls

from cesnet_service_path_plugin.views import (
    SegmentCircuitMappingDeleteView,
    SegmentCircuitMappingEditView,
    SegmentCircuitMappingListView,
    SegmentCircuitMappingView,
    SegmentDeleteView,
    SegmentEditView,
    SegmentListView,
    SegmentView,
    ServicePathDeleteView,
    ServicePathEditView,
    ServicePathListView,
    ServicePathSegmentMappingDeleteView,
    ServicePathSegmentMappingEditView,
    ServicePathSegmentMappingListView,
    ServicePathSegmentMappingView,
    ServicePathView,
    segment_geojson_download,
    segment_path_clear,
    segment_map_view,
    segment_geojson_api,
    SegmentsMapView,
    segments_map_api,
)

urlpatterns = (
    # Segment paths
    path("segments/", SegmentListView.as_view(), name="segment_list"),
    path("segments/add/", SegmentEditView.as_view(), name="segment_add"),
    path("segments/<int:pk>/", SegmentView.as_view(), name="segment"),
    path("segments/<int:pk>/edit/", SegmentEditView.as_view(), name="segment_edit"),
    path("segments/<int:pk>/delete/", SegmentDeleteView.as_view(), name="segment_delete"),
    # NEW: Map views
    path("segments/<int:pk>/map/", segment_map_view, name="segment_map"),
    path("segments/<int:pk>/geojson-api/", segment_geojson_api, name="segment_geojson_api"),
    # Adds Changelog, Journal, and Attachment tabs to the Segment view
    path(
        "segments/",
        include(get_model_urls("cesnet_service_path_plugin", "segment", detail=False)),
    ),
    path(
        "segments/<int:pk>/",
        include(get_model_urls("cesnet_service_path_plugin", "segment")),
    ),
    # GeoJSON download endpoint
    path("segments/<int:pk>/geojson/", segment_geojson_download, name="segment_geojson_download"),
    path("segments/<int:pk>/clear-path/", segment_path_clear, name="segment_path_clear"),
    # Segments map views
    path("segments/map/", SegmentsMapView.as_view(), name="segments_map"),
    path("segments/map/api/", segments_map_api, name="segments_map_api"),
    # ServicePath paths
    path("service-paths/", ServicePathListView.as_view(), name="servicepath_list"),
    path("service-paths/add/", ServicePathEditView.as_view(), name="servicepath_add"),
    path("service-paths/<int:pk>/", ServicePathView.as_view(), name="servicepath"),
    path(
        "service-paths/<int:pk>/edit/",
        ServicePathEditView.as_view(),
        name="servicepath_edit",
    ),
    path(
        "service-paths/<int:pk>/delete/",
        ServicePathDeleteView.as_view(),
        name="servicepath_delete",
    ),
    path(
        "service_paths/",
        include(get_model_urls("cesnet_service_path_plugin", "servicepath", detail=False)),
    ),
    path(
        "service_paths/<int:pk>/",
        include(get_model_urls("cesnet_service_path_plugin", "servicepath")),
    ),
    # ServicePathSegmentMapping paths
    path(
        "service-path-segment-mappings/",
        ServicePathSegmentMappingListView.as_view(),
        name="servicepathsegmentmapping_list",
    ),
    path(
        "service-path-segment-mappings/add/",
        ServicePathSegmentMappingEditView.as_view(),
        name="servicepathsegmentmapping_add",
    ),
    path(
        "service-path-segment-mappings/<int:pk>/",
        ServicePathSegmentMappingView.as_view(),
        name="servicepathsegmentmapping",
    ),
    path(
        "service-path-segment-mappings/<int:pk>/edit/",
        ServicePathSegmentMappingEditView.as_view(),
        name="servicepathsegmentmapping_edit",
    ),
    path(
        "service-path-segment-mappings/<int:pk>/delete/",
        ServicePathSegmentMappingDeleteView.as_view(),
        name="servicepathsegmentmapping_delete",
    ),
    path(
        "service-path-segment-mappings/",
        include(get_model_urls("cesnet_service_path_plugin", "servicepathsegmentmapping", detail=False)),
    ),
    path(
        "service-path-segment-mappings/<int:pk>/",
        include(get_model_urls("cesnet_service_path_plugin", "servicepathsegmentmapping")),
    ),
    # SegmentCircuitMapping paths
    path(
        "segment-circuit-mappings/",
        SegmentCircuitMappingListView.as_view(),
        name="segmentcircuitmapping_list",
    ),
    path(
        "segment-circuit-mappings/add/",
        SegmentCircuitMappingEditView.as_view(),
        name="segmentcircuitmapping_add",
    ),
    path(
        "segment-circuit-mappings/<int:pk>/",
        SegmentCircuitMappingView.as_view(),
        name="segmentcircuitmapping",
    ),
    path(
        "segment-circuit-mappings/<int:pk>/edit/",
        SegmentCircuitMappingEditView.as_view(),
        name="segmentcircuitmapping_edit",
    ),
    path(
        "segment-circuit-mappings/<int:pk>/delete/",
        SegmentCircuitMappingDeleteView.as_view(),
        name="segmentcircuitmapping_delete",
    ),
    path(
        "segment-circuit-mappings/",
        include(get_model_urls("cesnet_service_path_plugin", "segmentcircuitmapping", detail=False)),
    ),
    path(
        "segment-circuit-mappings/<int:pk>/",
        include(get_model_urls("cesnet_service_path_plugin", "segmentcircuitmapping")),
    ),
)



# Generated by Django 5.1.5 on 2025-02-10 08:16

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("cesnet_service_path_plugin", "0021_copy_service_path_state_to_status"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="servicepath",
            options={"ordering": ("name", "status", "kind")},
        ),
        migrations.RemoveField(
            model_name="servicepath",
            name="state",
        ),
    ]



from circuits.models import Circuit
from django.db import models
from django.urls import reverse
from netbox.models import NetBoxModel

from cesnet_service_path_plugin.models import Segment


class SegmentCircuitMapping(NetBoxModel):
    segment = models.ForeignKey(Segment, on_delete=models.CASCADE, null=False, blank=False)
    circuit = models.ForeignKey(Circuit, on_delete=models.CASCADE, null=False, blank=False)

    class Meta:
        ordering = ("segment", "circuit")
        unique_together = ("segment", "circuit")

    def __str__(self):
        return f"{self.segment} - {self.circuit}"

    def get_absolute_url(self):
        return reverse("plugins:cesnet_service_path_plugin:segmentcircuitmapping", args=[self.pk])



# Generated by Django 5.0.6 on 2024-07-25 11:10

from django.db import migrations


def copy_supplier_to_provider(apps, schema_editor):
    providers = apps.get_model("circuits", "Provider")
    segments = apps.get_model("cesnet_service_path_plugin", "Segment")

    for segment in segments.objects.all():
        tenant = segment.supplier
        try:
            provider = providers.objects.get(custom_field_data__tenant=tenant.id)
        except providers.DoesNotExist:
            raise Exception(
                f"Provider not found. Please ensure the tenant {tenant.id} - {tenant.name} has an associated provider."
            )
        segment.provider_id = provider.id
        segment.save()


class Migration(migrations.Migration):
    dependencies = [
        (
            "cesnet_service_path_plugin",
            "0009_rename_supplier_segment_contract_segment_provider_segment_contract_and_more",
        ),
    ]

    operations = [
        migrations.RunPython(copy_supplier_to_provider),
    ]



# Generated by Django 5.0.6 on 2024-06-03 07:43

import django.db.models.deletion
import taggit.managers
import utilities.json
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("circuits", "0043_circuittype_color"),
        ("extras", "0115_convert_dashboard_widgets"),
        (
            "cesnet_service_path_plugin",
            "0004_add_komora_id_and_imported_data_to_models",
        ),
    ]

    operations = [
        migrations.CreateModel(
            name="SegmentCircuitMapping",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                (
                    "circuit",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="circuits.circuit",
                    ),
                ),
                (
                    "segment",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="cesnet_service_path_plugin.segment",
                    ),
                ),
                (
                    "tags",
                    taggit.managers.TaggableManager(
                        through="extras.TaggedItem", to="extras.Tag"
                    ),
                ),
            ],
            options={
                "ordering": ("segment", "circuit"),
                "unique_together": {("segment", "circuit")},
            },
        ),
        migrations.AddField(
            model_name="segment",
            name="circuits",
            field=models.ManyToManyField(
                through="cesnet_service_path_plugin.SegmentCircuitMapping",
                to="circuits.circuit",
            ),
        ),
        migrations.CreateModel(
            name="ServicePathCircuitMapping",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                (
                    "circuit",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="circuits.circuit",
                    ),
                ),
                (
                    "service_path",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="cesnet_service_path_plugin.servicepath",
                    ),
                ),
                (
                    "tags",
                    taggit.managers.TaggableManager(
                        through="extras.TaggedItem", to="extras.Tag"
                    ),
                ),
            ],
            options={
                "ordering": ("service_path", "circuit"),
                "unique_together": {("service_path", "circuit")},
            },
        ),
        migrations.AddField(
            model_name="servicepath",
            name="circuits",
            field=models.ManyToManyField(
                through="cesnet_service_path_plugin.ServicePathCircuitMapping",
                to="circuits.circuit",
            ),
        ),
    ]
